<?php

namespace App\Controller;

use App\Entity\GeoContinents;
use App\Form\GeoContinentsType;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/geo/continents")
 */
class GeoContinentsController extends AbstractController
{
    /**
     * @Route("/", name="geo_continents_index", methods={"GET"})
     */
    public function index(): Response
    {
        $geoContinents = $this->getDoctrine()
            ->getRepository(GeoContinents::class)
            ->findAll();

        return $this->render('geo_continents/index.html.twig', [
            'geo_continents' => $geoContinents,
        ]);
    }

    /**
     * @Route("/new", name="geo_continents_new", methods={"GET","POST"})
     */
    public function new(Request $request): Response
    {
        $geoContinent = new GeoContinents();
        $form = $this->createForm(GeoContinentsType::class, $geoContinent);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->persist($geoContinent);
            $entityManager->flush();

            return $this->redirectToRoute('geo_continents_index');
        }

        return $this->render('geo_continents/new.html.twig', [
            'geo_continent' => $geoContinent,
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/{id}", name="geo_continents_show", methods={"GET"})
     */
    public function show(GeoContinents $geoContinent): Response
    {
        return $this->render('geo_continents/show.html.twig', [
            'geo_continent' => $geoContinent,
        ]);
    }

    /**
     * @Route("/{id}/edit", name="geo_continents_edit", methods={"GET","POST"})
     */
    public function edit(Request $request, GeoContinents $geoContinent): Response
    {
        $form = $this->createForm(GeoContinentsType::class, $geoContinent);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $this->getDoctrine()->getManager()->flush();

            return $this->redirectToRoute('geo_continents_index');
        }

        return $this->render('geo_continents/edit.html.twig', [
            'geo_continent' => $geoContinent,
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/{id}", name="geo_continents_delete", methods={"DELETE"})
     */
    public function delete(Request $request, GeoContinents $geoContinent): Response
    {
        if ($this->isCsrfTokenValid('delete'.$geoContinent->getId(), $request->request->get('_token'))) {
            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->remove($geoContinent);
            $entityManager->flush();
        }

        return $this->redirectToRoute('geo_continents_index');
    }
}
