<?php

namespace App\Controller;

use App\Entity\Station;
use App\Form\StationType;
use App\Repository\StationRepository;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use GeoIp2\Database\Reader;

/**
 * @Route("/station")
 */
class StationController extends AbstractController
{
    
    /**
     * @Route("/bypays", name="station_index_pays", methods={"GET"})
     */
    public function pays(StationRepository $stationRepository): Response
    {
        header("Access-Control-Allow-Origin: *");
        
	require_once('/home2/doulebtn/radio.douleb.tn/geoip2.phar');

        $reader = new Reader('/home2/doulebtn/radio.douleb.tn/GeoLite2-City.mmdb');
        $record = $reader->city($_SERVER['REMOTE_ADDR']);
        $cca2 = $record->country->isoCode;
	$stations = $stationRepository->findByPays($cca2);
        
        $jsonStations = json_encode($stations, JSON_UNESCAPED_UNICODE);
        return new Response($jsonStations);
    }
    
    /**
     * @Route("/xx", name="station_index_xx", methods={"GET"})
     */
    public function indexx(StationRepository $stationRepository): Response
    {
		$x = $stationRepository->findAll();
		foreach ($x as $value){
			if($value->getId() > 38) {
				$value->setSlug($value->getName());
				$entityManager = $this->getDoctrine()->getManager();
				$entityManager->persist($value);
				$entityManager->flush();
			    echo "Name : ".$value->getName(). " slug : ". $value->getSlug() ."</br>";
			}
		}
		
        return new Response("ok");
    }
	
	/**
     * @Route("/", name="station_index", methods={"GET"})
     */
    public function index(StationRepository $stationRepository): Response
    {
        return $this->render('station/index.html.twig', [
            'stations' => $stationRepository->findAll(),
        ]);
    }
	
	/**
     * @Route("/languages", name="lang", methods={"GET"})
     */
    public function languages(): Response
    {
		header("Access-Control-Allow-Origin: *");
		$browserLocale = '';
		$localePreferences = explode(",",$_SERVER['HTTP_ACCEPT_LANGUAGE']);
		if(is_array($localePreferences) && count($localePreferences) > 0) {
		$browserLocale = $localePreferences[0];
      //$_SESSION['browser_locale'] = $browserLocale;
	 }
	 $jsonStations = json_encode($browserLocale, JSON_UNESCAPED_UNICODE);
        return new Response($jsonStations);
		return new Response($browserLocale);
	}
	
    /**
     * @Route("/liste", name="stations", methods={"GET"})
     */
    public function stations(StationRepository $stationRepository): Response
    {
        header("Access-Control-Allow-Origin: *");

        $stations = $stationRepository->transformAll();
        $jsonStations = json_encode($stations, JSON_UNESCAPED_UNICODE);
        return new Response($jsonStations);
    }
	
	/**
     * @Route("/station/{slug}", name="station_one", methods={"GET"})
     */
    public function one(string $slug): Response
    {
		header("Access-Control-Allow-Origin: *");
		
        $station = $this->getDoctrine()->getManager()->getRepository(Station::class)->transformOne($slug);
        $jsonStation = json_encode($station, JSON_UNESCAPED_UNICODE);
        return new Response($jsonStation);
    }

    /**
     * @Route("/new", name="station_new", methods={"GET","POST"})
     */
    public function new(Request $request): Response
    {
        $station = new Station();
        $form = $this->createForm(StationType::class, $station);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            /** @var UploadedFile $brochureFile */
            $brochureFile = $form->get('image')->getData();

            // this condition is needed because the 'brochure' field is not required
            // so the PDF file must be processed only when a file is uploaded
            if ($brochureFile) {
                $originalFilename = pathinfo($brochureFile->getClientOriginalName(), PATHINFO_FILENAME);
                // this is needed to safely include the file name as part of the URL
                $safeFilename = transliterator_transliterate('Any-Latin; Latin-ASCII; [^A-Za-z0-9_] remove; Lower()', $originalFilename);
                $newFilename = $safeFilename.'-'.uniqid().'.'.$brochureFile->guessExtension();

                // Move the file to the directory where brochures are stored
                try {
                    $brochureFile->move(
                        $this->getParameter('brochures_directory'),
                        $newFilename
                    );
                } catch (FileException $e) {
                    // ... handle exception if something happens during file upload
                }

                // updates the 'brochureFilename' property to store the PDF file name
                // instead of its contents
                $station->setImage($newFilename);
            }

            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->persist($station);
            $entityManager->flush();

            return $this->redirectToRoute('station_new');
        }

        return $this->render('station/new.html.twig', [
            'station' => $station,
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/{slug}/show", name="station_show", methods={"GET"})
     */
    public function show(Station $station): Response
    {
        return $this->render('station/show.html.twig', [
            'station' => $station,
        ]);
    }

    /**
     * @Route("/{id}/edit", name="station_edit", methods={"GET","POST"})
     */
    public function edit(Request $request, Station $station): Response
    {
        $form = $this->createForm(StationType::class, $station);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $this->getDoctrine()->getManager()->flush();

            return $this->redirectToRoute('station_index');
        }

        return $this->render('station/edit.html.twig', [
            'station' => $station,
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/{id}/delete", name="station_delete", methods={"DELETE"})
     */
    public function delete(Request $request, Station $station): Response
    {
        if ($this->isCsrfTokenValid('delete'.$station->getId(), $request->request->get('_token'))) {
            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->remove($station);
            $entityManager->flush();
        }

        return $this->redirectToRoute('station_index');
    }
}
